@@ -160,6 +160,10 @@ katze_load_cached_icon               (const gchar*    uri,
 GtkWidget*
 katze_uri_entry_new                  (GtkWidget*      other_widget);
 
+void
+katze_widget_add_class               (GtkWidget*      widget,
+                                      const gchar*    class_name);
+
 void
 katze_assert_str_equal               (const gchar*    input,
                                       const gchar*    result,
