@@ -14,6 +14,7 @@
 
 #include <katze/katze.h>
 #include "midori-platform.h"
+#include "midori-core.h"
 #include <glib/gi18n.h>
 
 G_DEFINE_TYPE (MidoriExtension, midori_extension, G_TYPE_OBJECT);
@@ -317,10 +318,10 @@ midori_extension_activate_cb (MidoriExtension* extension,
                         strlen (filename) - strlen ("." G_MODULE_SUFFIX));
                 else
                     filename = g_strdup (filename);
-                folder = g_strconcat ("extensions/", filename, NULL);
+                folder = g_build_filename ("extensions", filename, NULL);
                 g_free (filename);
                 katze_assign (config_file,
-                    sokoke_find_config_filename (folder, "config"));
+                    midori_paths_get_preset_filename (folder, "config"));
                 g_free (folder);
                 g_key_file_load_from_file (extension->priv->key_file, config_file,
                                            G_KEY_FILE_KEEP_COMMENTS, NULL);
@@ -340,29 +341,32 @@ midori_extension_activate_cb (MidoriExtension* extension,
         if (setting->type == G_TYPE_BOOLEAN)
         {
             MESettingBoolean* setting_ = (MESettingBoolean*)setting;
-            if (extension->priv->key_file)
-                setting_->value = sokoke_key_file_get_boolean_default (
-                    extension->priv->key_file,
-                    "settings", setting->name, setting_->default_value, NULL);
+            if (extension->priv->key_file
+             && g_key_file_has_key (extension->priv->key_file, "settings", setting_->name, NULL))
+                setting_->value = g_key_file_get_boolean (extension->priv->key_file,
+                    "settings", setting->name, NULL);
             else
                 setting_->value = setting_->default_value;
         }
         else if (setting->type == G_TYPE_INT)
         {
             MESettingInteger* setting_ = (MESettingInteger*)setting;
-            if (extension->priv->key_file)
-                setting_->value = sokoke_key_file_get_integer_default (
-                    extension->priv->key_file,
-                    "settings", setting->name, setting_->default_value, NULL);
+            if (extension->priv->key_file
+             && g_key_file_has_key (extension->priv->key_file, "settings", setting_->name, NULL))
+                setting_->value = g_key_file_get_integer (extension->priv->key_file,
+                    "settings", setting_->name, NULL);
             else
                 setting_->value = setting_->default_value;
         }
         else if (setting->type == G_TYPE_STRING)
         {
             if (extension->priv->key_file)
-                setting->value = sokoke_key_file_get_string_default (
-                    extension->priv->key_file,
-                    "settings", setting->name, setting->default_value, NULL);
+            {
+                setting->value = g_key_file_get_string (
+                    extension->priv->key_file, "settings", setting->name, NULL);
+                if (setting->value == NULL)
+                    setting->value = setting->default_value;
+            }
             else
                 setting->value = g_strdup (setting->default_value);
         }
@@ -371,10 +375,10 @@ midori_extension_activate_cb (MidoriExtension* extension,
             MESettingStringList* setting_ = (MESettingStringList*)setting;
             if (extension->priv->key_file)
             {
-                setting_->value = sokoke_key_file_get_string_list_default (
-                    extension->priv->key_file,
-                    "settings", setting->name, &setting_->length,
-                    setting_->default_value, &setting_->default_length, NULL);
+                setting_->value = g_key_file_get_string_list (extension->priv->key_file,
+                    "settings", setting->name, &setting_->length, NULL);
+                if (setting_->value == NULL)
+                    setting_->value = g_strdupv (setting_->default_value);
             }
             else
                 setting_->value = g_strdupv (setting_->default_value);
@@ -645,10 +649,11 @@ midori_extension_get_config_dir (MidoriExtension* extension)
     if (!extension->priv->config_dir)
     {
         gchar* filename = g_object_get_data (G_OBJECT (extension), "filename");
-        if (!filename)
-            return "/";
-        extension->priv->config_dir = g_build_filename (
-            sokoke_set_config_dir (NULL), "extensions", filename, NULL);
+        if (filename != NULL)
+            extension->priv->config_dir = g_build_filename (
+                midori_paths_get_config_dir (), "extensions", filename, NULL);
+        else
+            extension->priv->config_dir = NULL;
     }
 
     return extension->priv->config_dir;
