@@ -376,10 +376,14 @@ midori_location_action_add_search_engines (MidoriLocationAction* action,
 {
     KatzeItem* item;
     gint i = 0;
+    #ifndef G_OS_WIN32
     GtkStyle* style;
+    #endif
 
     gtk_widget_realize (action->treeview);
+    #ifndef G_OS_WIN32
     style = gtk_widget_get_style (action->treeview);
+    #endif
 
     /* FIXME: choose 3 most frequently except for default */
     KATZE_ARRAY_FOREACH_ITEM (item, action->search_engines)
@@ -397,7 +401,9 @@ midori_location_action_add_search_engines (MidoriLocationAction* action,
         icon = midori_search_action_get_icon (item, action->treeview, NULL, FALSE);
         gtk_list_store_insert_with_values (store, NULL, matches + i,
             URI_COL, uri, TITLE_COL, desc, YALIGN_COL, 0.25,
+            #ifndef G_OS_WIN32
             BACKGROUND_COL, style ? &style->bg[GTK_STATE_NORMAL] : NULL,
+            #endif
             STYLE_COL, 1, FAVICON_COL, icon, -1);
         g_free (uri);
         g_free (title);
@@ -409,9 +415,11 @@ midori_location_action_add_search_engines (MidoriLocationAction* action,
         if (i > 2 && matches > 0)
         {
             gtk_list_store_insert_with_values (store, NULL, matches + i,
-                URI_COL, "about:search", TITLE_COL, _("Search with..."),
+                URI_COL, "about:search", TITLE_COL, _("Search with…"),
                 YALIGN_COL, 0.25,
+                #ifndef G_OS_WIN32
                 BACKGROUND_COL, style ? &style->bg[GTK_STATE_NORMAL] : NULL,
+                #endif
                 STYLE_COL, 1, FAVICON_COL, NULL, -1);
             i++;
             break;
@@ -814,15 +822,31 @@ midori_location_action_activate (GtkAction* action)
         GTK_ACTION_CLASS (midori_location_action_parent_class)->activate (action);
 }
 
+static void
+midori_location_action_entry_drag_data_get_cb (GtkWidget*        entry,
+                                               GdkDragContext*   context,
+                                               GtkSelectionData* data,
+                                               guint             info,
+                                               guint32           time,
+                                               GtkAction*        action)
+{
+    if (gtk_entry_get_current_icon_drag_source (GTK_ENTRY (entry)) == GTK_ENTRY_ICON_PRIMARY)
+    {
+        const gchar* uri = gtk_entry_get_text (GTK_ENTRY (entry));
+        gchar** uris = g_strsplit (uri, uri, 1);
+        gtk_selection_data_set_uris (data, uris);
+        g_strfreev (uris);
+    }
+}
+
 static GtkWidget*
 midori_location_action_create_tool_item (GtkAction* action)
 {
     GtkWidget* toolitem;
     GtkWidget* alignment;
     GtkWidget* entry;
-    #if HAVE_HILDON
-    HildonGtkInputMode mode;
-    #endif
+
+    GtkTargetList *targetlist;
 
     toolitem = GTK_WIDGET (gtk_tool_item_new ());
     gtk_tool_item_set_expand (GTK_TOOL_ITEM (toolitem), TRUE);
@@ -831,46 +855,21 @@ midori_location_action_create_tool_item (GtkAction* action)
     gtk_widget_show (alignment);
     gtk_container_add (GTK_CONTAINER (toolitem), alignment);
 
-    #if HAVE_HILDON
-    entry = gtk_entry_new ();
-    mode = hildon_gtk_entry_get_input_mode (GTK_ENTRY (entry));
-    mode &= ~HILDON_GTK_INPUT_MODE_AUTOCAP;
-    hildon_gtk_entry_set_input_mode (GTK_ENTRY (entry), mode);
-    #else
     entry = gtk_icon_entry_new ();
-    /* Work-around icon being activatable by default */
     gtk_icon_entry_set_icon_highlight (GTK_ICON_ENTRY (entry),
-         GTK_ICON_ENTRY_PRIMARY, FALSE);
+         GTK_ICON_ENTRY_PRIMARY, TRUE);
     gtk_icon_entry_set_icon_highlight (GTK_ICON_ENTRY (entry),
          GTK_ICON_ENTRY_SECONDARY, TRUE);
-    #endif
+
+    targetlist = gtk_target_list_new (NULL, 0);
+    gtk_target_list_add_uri_targets (targetlist, 0);
+    gtk_entry_set_icon_drag_source (GTK_ENTRY (entry), GTK_ENTRY_ICON_PRIMARY, targetlist, GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
+    gtk_target_list_unref (targetlist);
+    g_signal_connect (entry, "drag-data-get",
+        G_CALLBACK (midori_location_action_entry_drag_data_get_cb), action);
     gtk_widget_show (entry);
     gtk_container_add (GTK_CONTAINER (alignment), entry);
 
-    #if GTK_CHECK_VERSION (3, 0, 0)
-    {
-    static const gchar default_style[] =
-        ".security_unknown {\n"
-        "background-image: none;\n"
-        "background-color: #ef7070;\n"
-        "color: #000;\n"
-        "}\n"
-        ".security_trusted {\n"
-        "background-image: none;\n"
-        "background-color: #d1eeb9;\n"
-        "color: #000;\n"
-        "}\n";
-    GtkCssProvider* css_provider;
-    GtkStyleContext* context;
-
-    css_provider = gtk_css_provider_new ();
-    context = gtk_widget_get_style_context (entry);
-    gtk_css_provider_load_from_data (css_provider, default_style, -1, NULL);
-    gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (css_provider),
-                                    GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
-    }
-    #endif
-
     return toolitem;
 }
 
@@ -993,10 +992,6 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
         /* Return FALSE to allow Escape to stop loading */
         return FALSE;
     }
-    case GDK_KEY_Page_Up:
-    case GDK_KEY_Page_Down:
-        if (!(location_action->popup && gtk_widget_get_visible (location_action->popup)))
-            return TRUE;
     case GDK_KEY_Delete:
     case GDK_KEY_KP_Delete:
     {
@@ -1045,7 +1040,12 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
     case GDK_KEY_KP_Up:
     case GDK_KEY_Tab:
     case GDK_KEY_ISO_Left_Tab:
+    case GDK_KEY_Page_Down:
+    case GDK_KEY_Page_Up:
     {
+        if ((event->keyval == GDK_KEY_Page_Up || event->keyval == GDK_KEY_Page_Down) &&
+           !(location_action->popup && gtk_widget_get_visible (location_action->popup)))
+            return TRUE;
         if (location_action->popup && gtk_widget_get_visible (location_action->popup))
         {
             GtkTreeModel* model = location_action->completion_model;
@@ -1055,35 +1055,62 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
             gint selected = location_action->completion_index;
 
             if (event->keyval == GDK_KEY_Down || event->keyval == GDK_KEY_KP_Down
-             || event->keyval == GDK_KEY_Tab  || event->keyval == GDK_KEY_ISO_Left_Tab)
-                selected = MIN (selected + 1, matches -1);
-            else if (event->keyval == GDK_KEY_Up || event->keyval == GDK_KEY_KP_Up)
+             || ((event->keyval == GDK_KEY_Tab  || event->keyval == GDK_KEY_ISO_Left_Tab)
+             && !(event->state & GDK_SHIFT_MASK)))
+            {
+                selected = selected + 1;
+                if (selected == matches)
+                    selected = -1;
+            }
+            else if (event->keyval == GDK_KEY_Up || event->keyval == GDK_KEY_KP_Up
+                  || ((event->keyval == GDK_KEY_Tab  || event->keyval == GDK_KEY_ISO_Left_Tab)
+                  && (event->state & GDK_SHIFT_MASK)))
             {
                 if (selected == -1)
                     selected = matches - 1;
                 else
-                    selected = MAX (selected - 1, 0);
+                    selected = selected - 1;
             }
             else if (event->keyval == GDK_KEY_Page_Down)
-                selected = MIN (selected + 14, matches -1);
+            {
+                if (selected == -1)
+                    selected = 0;
+                else if (selected < matches - 1)
+                    selected = MIN (selected + 14, matches -1);
+                else
+                    selected = -1;
+            }
             else if (event->keyval == GDK_KEY_Page_Up)
-                selected = MAX (selected - 14, 0);
+            {
+                if (selected == -1)
+                    selected = matches - 1;
+                else if (selected > 0)
+                    selected = MAX (selected - 14, 0);
+                else
+                    selected = -1;
+            }
             else if (event->keyval != GDK_KEY_KP_Delete && event->keyval != GDK_KEY_Delete)
                 g_assert_not_reached ();
 
-            path = gtk_tree_path_new_from_indices (selected, -1);
-            gtk_tree_view_set_cursor (GTK_TREE_VIEW (location_action->treeview),
-                                      path, NULL, FALSE);
-            gtk_tree_path_free (path);
-
-            if (gtk_tree_model_iter_nth_child (model, &iter, NULL, selected))
+            if (selected != -1)
             {
-                gchar* uri;
-                gtk_tree_model_get (model, &iter, URI_COL, &uri, -1);
-                /* Update the layout without actually changing the text */
-                pango_layout_set_text (gtk_entry_get_layout (entry), uri, -1);
-                g_free (uri);
+                path = gtk_tree_path_new_from_indices (selected, -1);
+                gtk_tree_view_set_cursor (GTK_TREE_VIEW (location_action->treeview),
+                                          path, NULL, FALSE);
+                gtk_tree_path_free (path);
+
+                if (gtk_tree_model_iter_nth_child (model, &iter, NULL, selected))
+                {
+                    gchar* uri;
+                    gtk_tree_model_get (model, &iter, URI_COL, &uri, -1);
+                    /* Update the layout without actually changing the text */
+                    pango_layout_set_text (gtk_entry_get_layout (entry), uri, -1);
+                    g_free (uri);
+                }
             }
+            else
+                gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (location_action->treeview)));
+
             location_action->completion_index = selected;
             return TRUE;
         }
@@ -1147,12 +1174,237 @@ midori_location_action_focus_out_event_cb (GtkWidget*   widget,
     return FALSE;
 }
 
+#ifdef HAVE_GCR
+    #define GCR_API_SUBJECT_TO_CHANGE
+    #include <gcr/gcr.h>
+#endif
+
+#if defined (HAVE_LIBSOUP_2_34_0)
+static GHashTable* message_map = NULL;
+void
+midori_map_add_message (SoupMessage* message)
+{
+    SoupURI* uri = soup_message_get_uri (message);
+    if (message_map == NULL)
+        message_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+    g_return_if_fail (uri && uri->host);
+    g_hash_table_insert (message_map, g_strdup (uri->host), g_object_ref (message));
+}
+
+SoupMessage*
+midori_map_get_message (SoupMessage* message)
+{
+    SoupURI* uri = soup_message_get_uri (message);
+    SoupMessage* full;
+    g_return_val_if_fail (uri && uri->host, message);
+    full = g_hash_table_lookup (message_map, uri->host);
+    g_return_val_if_fail (full, message);
+    return full;
+}
+
+#ifdef HAVE_GCR
+typedef enum {
+    MIDORI_CERT_TRUST,
+    MIDORI_CERT_REVOKE,
+    MIDORI_CERT_EXPORT,
+} MidoriCertTrust;
+
+static void
+midori_location_action_cert_response_cb (GtkWidget*      dialog,
+                                         gint            response,
+                                         GcrCertificate* gcr_cert)
+{
+    gchar* peer = g_object_get_data (G_OBJECT (gcr_cert), "peer");
+    GError* error = NULL;
+    if (response == MIDORI_CERT_TRUST)
+        gcr_trust_add_pinned_certificate (gcr_cert, GCR_PURPOSE_SERVER_AUTH, peer, NULL, &error);
+    else if (response == MIDORI_CERT_REVOKE)
+        gcr_trust_remove_pinned_certificate (gcr_cert, GCR_PURPOSE_SERVER_AUTH, peer, NULL, &error);
+    else if (response == MIDORI_CERT_EXPORT)
+    {
+        /* FIXME: Would be nice if GcrCertificateExporter became public */
+        gchar* filename = g_strconcat (peer, ".crt", NULL);
+        GtkWidget* dialog = sokoke_file_chooser_dialog_new (_("Export certificate"),
+            NULL, GTK_FILE_CHOOSER_ACTION_SAVE);
+        gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+        gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), filename);
+        g_free (filename);
+
+        if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+        {
+            gsize n_data;
+            gconstpointer data = gcr_certificate_get_der_data (gcr_cert, &n_data);
+            g_return_if_fail (data);
+            filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+            g_file_set_contents (filename, data, n_data, NULL);
+            g_free (filename);
+        }
+        gtk_widget_destroy (dialog);
+    }
+    if (error != NULL)
+    {
+        g_warning ("Error %s trust: %s", response == MIDORI_CERT_TRUST ?
+                   "granting" : "revoking", error->message);
+        g_error_free (error);
+    }
+    gtk_widget_destroy (dialog);
+}
+#endif
+
+const gchar*
+midori_location_action_tls_flags_to_string (GTlsCertificateFlags tls_flags)
+{
+    const gchar* tls_error;
+    if (tls_flags & G_TLS_CERTIFICATE_UNKNOWN_CA)
+        tls_error = _("The signing certificate authority is not known.");
+    else if (tls_flags & G_TLS_CERTIFICATE_BAD_IDENTITY)
+        tls_error = _("The certificate does not match the expected identity of the site that it was retrieved from.");
+    else if(tls_flags & G_TLS_CERTIFICATE_NOT_ACTIVATED)
+        tls_error = _("The certificate's activation time is still in the future.");
+    else if (tls_flags & G_TLS_CERTIFICATE_EXPIRED)
+        tls_error = _("The certificate has expired");
+    else if (tls_flags & G_TLS_CERTIFICATE_REVOKED)
+        tls_error = _("The certificate has been revoked according to the GTlsConnection's certificate revocation list.");
+    else if (tls_flags & G_TLS_CERTIFICATE_INSECURE)
+        tls_error = _("The certificate's algorithm is considered insecure.");
+    else if (tls_flags & G_TLS_CERTIFICATE_GENERIC_ERROR)
+        tls_error = _("Some other error occurred validating the certificate.");
+    else
+        tls_error = "Unknown GTLSCertificateFlags value";
+    return tls_error;
+}
+
+void
+midori_location_action_show_page_info (GtkWidget* widget,
+                                       GtkBox*    box,
+                                       GtkWidget* dialog)
+{
+    MidoriBrowser* browser = midori_browser_get_for_widget (widget);
+    MidoriView* view = MIDORI_VIEW (midori_browser_get_current_tab (browser));
+    WebKitWebView* web_view = WEBKIT_WEB_VIEW (midori_view_get_web_view (view));
+    WebKitWebFrame* web_frame = webkit_web_view_get_main_frame (web_view);
+    WebKitWebDataSource* source = webkit_web_frame_get_data_source (web_frame);
+    WebKitNetworkRequest* request = webkit_web_data_source_get_request (source);
+    SoupMessage* message = midori_map_get_message (webkit_network_request_get_message (request));
+    GTlsCertificate* tls_cert;
+    GTlsCertificateFlags tls_flags;
+
+    g_return_if_fail (message);
+    g_object_get (message, "tls-certificate", &tls_cert, "tls-errors", &tls_flags, NULL);
+
+    if (tls_cert == NULL)
+        return;
+
+    #ifdef HAVE_GCR
+    GByteArray* der_cert;
+    GcrCertificate* gcr_cert;
+    GtkWidget* details;
+    SoupURI* uri = soup_message_get_uri (message);
+
+    g_object_get (tls_cert, "certificate", &der_cert, NULL);
+    gcr_cert = gcr_simple_certificate_new (
+        der_cert->data, der_cert->len);
+    g_byte_array_unref (der_cert);
+    details = (GtkWidget*)gcr_certificate_details_widget_new (gcr_cert);
+    gtk_widget_show (details);
+    gtk_container_add (GTK_CONTAINER (box), details);
+    if (gcr_trust_is_certificate_pinned (gcr_cert, GCR_PURPOSE_SERVER_AUTH, uri->host, NULL, NULL))
+        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+            ("_Don't trust this website"), MIDORI_CERT_REVOKE, NULL);
+    else if (tls_flags > 0)
+        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+            ("_Trust this website"), MIDORI_CERT_TRUST, NULL);
+    gtk_container_child_set (GTK_CONTAINER (gtk_dialog_get_action_area (GTK_DIALOG (dialog))),
+        gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Export certificate"), MIDORI_CERT_EXPORT),
+        "secondary", TRUE, NULL);
+
+    g_object_set_data_full (G_OBJECT (gcr_cert), "peer", g_strdup (uri->host), (GDestroyNotify)g_free);
+    g_object_set_data_full (G_OBJECT (dialog), "gcr-cert", gcr_cert, (GDestroyNotify)g_object_unref);
+    g_signal_connect (dialog, "response",
+        G_CALLBACK (midori_location_action_cert_response_cb), gcr_cert);
+    /* With GTK+2 the scrolled contents can't communicate a natural size to the window */
+    #if !GTK_CHECK_VERSION (3, 0, 0)
+    gtk_window_set_default_size (GTK_WINDOW (dialog), 250, 200);
+    #endif
+    #else
+    const gchar* tls_error = midori_location_action_tls_flags_to_string (tls_flags);
+
+    if (!g_tls_certificate_get_issuer (tls_cert))
+        gtk_box_pack_start (box, gtk_label_new (_("Self-signed")), FALSE, FALSE, 0);
+
+    gtk_box_pack_start (box, gtk_label_new (tls_error), FALSE, FALSE, 0);
+    #endif
+
+    g_object_unref (tls_cert);
+}
+#endif
+
+#ifndef HAVE_GRANITE
+static gboolean
+midori_location_action_dialog_focus_out_cb (GtkWidget* dialog,
+                                            GdkEvent*  event,
+                                            gpointer   user_data)
+{
+    gtk_widget_destroy (dialog);
+    return TRUE;
+}
+#endif
+
 static void
 midori_location_action_icon_released_cb (GtkWidget*           widget,
                                          GtkIconEntryPosition icon_pos,
                                          gint                 button,
                                          GtkAction*           action)
 {
+    /* The dialog should "toggle" like a menu, as far as users go
+       FIXME: Half-working: the dialog closes but re-opens */
+    static GtkWidget* dialog = NULL;
+    if (icon_pos == GTK_ICON_ENTRY_PRIMARY && dialog != NULL)
+        gtk_widget_destroy (dialog);
+
+    if (icon_pos == GTK_ICON_ENTRY_PRIMARY)
+    {
+        const gchar* title = _("Security details");
+        GtkWidget* content_area;
+        GtkWidget* hbox;
+        #ifdef HAVE_GRANITE
+        gint root_x, root_y;
+        GdkRectangle icon_rect;
+        /* FIXME: granite: should return GtkWidget* like GTK+ */
+        dialog = (GtkWidget*)granite_widgets_pop_over_new ();
+        gchar* markup = g_strdup_printf ("<b>%s</b>", title);
+        GtkWidget* label = gtk_label_new (markup);
+        content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+        g_free (markup);
+        gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+        gtk_box_pack_start (GTK_BOX (content_area), label, FALSE, FALSE, 0);
+        gtk_entry_get_icon_area (GTK_ENTRY (widget), icon_pos, &icon_rect);
+        gdk_window_get_root_coords (gtk_widget_get_window (widget),
+            icon_rect.x + icon_rect.width / 2, icon_rect.y + icon_rect.height,
+            &root_x, &root_y);
+        granite_widgets_pop_over_move_to_coords (GRANITE_WIDGETS_POP_OVER (dialog),
+            root_x, root_y, TRUE);
+        #else
+        dialog = gtk_dialog_new_with_buttons (title, GTK_WINDOW (gtk_widget_get_toplevel (widget)),
+            GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR, NULL, NULL);
+        /* FIXME: check focus-in on the transient-for window instead of
+                  focus-out-event */
+        g_signal_connect (dialog, "focus-out-event",
+            G_CALLBACK (midori_location_action_dialog_focus_out_cb), NULL);
+        content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+        #endif
+        hbox = gtk_hbox_new (FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (hbox), gtk_image_new_from_gicon (
+            gtk_entry_get_icon_gicon (GTK_ENTRY (widget), icon_pos), GTK_ICON_SIZE_DIALOG), FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (hbox),
+            gtk_label_new (gtk_icon_entry_get_tooltip (GTK_ICON_ENTRY (widget), icon_pos)), FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, 0);
+        #if defined (HAVE_LIBSOUP_2_34_0)
+        midori_location_action_show_page_info (widget, GTK_BOX (content_area), dialog);
+        #endif
+        g_signal_connect (dialog, "destroy", G_CALLBACK (gtk_widget_destroyed), &dialog);
+        gtk_widget_show_all (dialog);
+    }
     if (icon_pos == GTK_ICON_ENTRY_SECONDARY)
     {
         gboolean result;
@@ -1711,91 +1963,30 @@ midori_location_action_set_security_hint (MidoriLocationAction* location_action,
     for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
     {
-        #if !GTK_CHECK_VERSION (3, 0, 0)
-        const gchar* bg_color = NULL;
-        const gchar* fg_color = NULL;
-        #endif
         GtkWidget* entry = midori_location_action_entry_for_proxy (proxies->data);
-        GdkScreen* screen = gtk_widget_get_screen (entry);
-        GtkIconTheme* icon_theme = gtk_icon_theme_get_for_screen (screen);
 
         if (hint == MIDORI_SECURITY_UNKNOWN)
         {
-            #if !GTK_CHECK_VERSION (3, 0, 0)
-            bg_color = "#ef7070";
-            fg_color = "#000";
-            #endif
-            #if !HAVE_HILDON
-            if (gtk_icon_theme_has_icon (icon_theme, "channel-insecure-symbolic"))
-                gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, "channel-insecure-symbolic");
-            else if (gtk_icon_theme_has_icon (icon_theme, "lock-insecure"))
-                gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, "lock-insecure");
-            else
-                gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, GTK_STOCK_INFO);
+            gchar* icon_names[] = { "channel-insecure-symbolic", "lock-insecure", "dialog-information", NULL };
+            gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry), GTK_ICON_ENTRY_PRIMARY,
+                g_themed_icon_new_from_names (icon_names, -1));
             gtk_icon_entry_set_tooltip (GTK_ICON_ENTRY (entry),
                 GTK_ICON_ENTRY_PRIMARY, _("Not verified"));
-            #endif
         }
         else if (hint == MIDORI_SECURITY_TRUSTED)
         {
-            #if !GTK_CHECK_VERSION (3, 0, 0)
-            bg_color = "#d1eeb9";
-            fg_color = "#000";
-            #endif
-            #if !HAVE_HILDON
-            if (gtk_icon_theme_has_icon (icon_theme, "channel-secure-symbolic"))
-                gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, "channel-secure-symbolic");
-            else if (gtk_icon_theme_has_icon (icon_theme, "lock-secure"))
-                gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, "lock-secure");
-            else
-                gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, GTK_STOCK_DIALOG_AUTHENTICATION);
+            gchar* icon_names[] = { "channel-secure-symbolic", "lock-secure", "locked", NULL };
+            gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry), GTK_ICON_ENTRY_PRIMARY,
+                g_themed_icon_new_from_names (icon_names, -1));
             gtk_icon_entry_set_tooltip (GTK_ICON_ENTRY (entry),
                 GTK_ICON_ENTRY_PRIMARY, _("Verified and encrypted connection"));
-            #endif
         }
         else if (hint == MIDORI_SECURITY_NONE)
         {
-            if (gtk_icon_theme_has_icon (icon_theme, "text-html-symbolic"))
-                gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, "text-html-symbolic");
-            else
-                gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
-                    GTK_ICON_ENTRY_PRIMARY, "text-html");
+            gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry), GTK_ICON_ENTRY_PRIMARY,
+                g_themed_icon_new_with_default_fallbacks ("text-html-symbolic"));
             gtk_icon_entry_set_tooltip (GTK_ICON_ENTRY (entry),
                 GTK_ICON_ENTRY_PRIMARY, _("Open, unencrypted connection"));
         }
-
-        {
-        #if GTK_CHECK_VERSION (3, 0, 0)
-        GtkStyleContext* context = gtk_widget_get_style_context (entry);
-        if (hint == MIDORI_SECURITY_UNKNOWN)
-        {
-            gtk_style_context_add_class (context, "security_unknown");
-            gtk_style_context_remove_class (context, "security_trusted");
-        }
-        else if (hint == MIDORI_SECURITY_TRUSTED)
-        {
-            gtk_style_context_add_class (context, "security_trusted");
-            gtk_style_context_remove_class (context, "security_unknown");
-        }
-        else if (hint == MIDORI_SECURITY_NONE)
-        {
-            gtk_style_context_remove_class (context, "security_unknown");
-            gtk_style_context_remove_class (context, "security_trusted");
-        }
-        #else
-        GdkColor color = { 0 };
-        if (bg_color) gdk_color_parse (bg_color, &color);
-        gtk_widget_modify_base (entry, GTK_STATE_NORMAL, bg_color ? &color : NULL);
-        if (fg_color) gdk_color_parse (fg_color, &color);
-        gtk_widget_modify_text (entry, GTK_STATE_NORMAL, fg_color ? &color : NULL);
-        #endif
-        }
     }
 }
