@@ -16,7 +16,7 @@
 #include "midori-browser.h"
 #include "midori-platform.h"
 #include "midori-view.h"
-#include "midori-viewable.h"
+#include "midori-core.h"
 
 #include <glib/gi18n.h>
 #include <string.h>
@@ -120,66 +120,87 @@ midori_bookmarks_get_stock_id (MidoriViewable* viewable)
     return STOCK_BOOKMARKS;
 }
 
+/* TODO: Function never used */
 void
-midori_bookmarks_export_array_db (sqlite3*     db,
-                                  KatzeArray*  array,
-                                  const gchar* folder)
+midori_bookmarks_export_array_db (sqlite3*    db,
+                                  KatzeArray* array,
+                                  gint64      parentid)
 {
     KatzeArray* root_array;
     KatzeArray* subarray;
     KatzeItem* item;
     GList* list;
+    gchar* parent_id;
 
-    if (!(root_array = midori_array_query (array, "*", "folder='%q'", folder)))
+    parent_id = g_strdup_printf ("%" G_GINT64_FORMAT, parentid);
+    if (!(root_array = midori_array_query (array, "*", "parentid = %q", parent_id)))
+    {
+        g_free (parent_id);
         return;
+    }
     KATZE_ARRAY_FOREACH_ITEM_L (item, root_array, list)
     {
         if (KATZE_ITEM_IS_FOLDER (item))
         {
             subarray = katze_array_new (KATZE_TYPE_ARRAY);
             katze_item_set_name (KATZE_ITEM (subarray), katze_item_get_name (item));
-            midori_bookmarks_export_array_db (db, subarray, katze_item_get_name (item));
+            midori_bookmarks_export_array_db (db, subarray,
+                    katze_item_get_meta_integer (item, "parentid"));
             katze_array_add_item (array, subarray);
         }
         else
             katze_array_add_item (array, item);
     }
+
+    g_free (parent_id);
     g_list_free (list);
 }
 
 void
-midori_bookmarks_import_array_db (sqlite3*     db,
-                                  KatzeArray*  array,
-                                  const gchar* folder)
+midori_bookmarks_import_array_db (sqlite3*    db,
+                                  KatzeArray* array,
+                                  gint64      parentid)
 {
     GList* list;
     KatzeItem* item;
+    gint64 id;
 
     if (!db)
         return;
 
     KATZE_ARRAY_FOREACH_ITEM_L (item, array, list)
     {
+        id = midori_bookmarks_insert_item_db (db, item, parentid);
         if (KATZE_IS_ARRAY (item))
-            midori_bookmarks_import_array_db (db, KATZE_ARRAY (item), folder);
-        midori_bookmarks_insert_item_db (db, item, folder);
+            midori_bookmarks_import_array_db (db, KATZE_ARRAY (item), id);
     }
     g_list_free (list);
 }
 
 static KatzeArray*
 midori_bookmarks_read_from_db (MidoriBookmarks* bookmarks,
-                               const gchar*     folder,
+                               gint64           parentid,
                                const gchar*     keyword)
 {
     KatzeArray* array;
 
     if (keyword && *keyword)
         array = midori_array_query (bookmarks->array,
-           "uri, title, desc, app, toolbar, folder", "title LIKE '%%%q%%'", keyword);
+           "id, parentid, title, uri, desc, app, toolbar, pos_panel, pos_bar", "title LIKE '%%%q%%'", keyword);
     else
-        array = midori_array_query (bookmarks->array,
-           "uri, title, desc, app, toolbar, folder", "folder = '%q'", folder);
+    {
+        if (parentid > 0)
+        {
+            gchar* parent_id = g_strdup_printf ("%" G_GINT64_FORMAT, parentid);
+            array = midori_array_query (bookmarks->array,
+               "id, parentid, title, uri, desc, app, toolbar, pos_panel, pos_bar", "parentid = %q", parent_id);
+
+            g_free (parent_id);
+        }
+        else
+            array = midori_array_query (bookmarks->array,
+               "id, parentid, title, uri, desc, app, toolbar, pos_panel, pos_bar", "parentid IS NULL", NULL);
+    }
     return array ? array : katze_array_new (KATZE_TYPE_ITEM);
 }
 
@@ -187,7 +208,7 @@ static void
 midori_bookmarks_read_from_db_to_model (MidoriBookmarks* bookmarks,
                                         GtkTreeStore*    model,
                                         GtkTreeIter*     parent,
-                                        const gchar*     folder,
+                                        gint64           parentid,
                                         const gchar*     keyword)
 {
     KatzeArray* array;
@@ -195,7 +216,7 @@ midori_bookmarks_read_from_db_to_model (MidoriBookmarks* bookmarks,
     KatzeItem* item;
     GtkTreeIter child;
 
-    array = midori_bookmarks_read_from_db (bookmarks, folder, keyword);
+    array = midori_bookmarks_read_from_db (bookmarks, parentid, keyword);
     katze_bookmark_populate_tree_view (array, model, parent);
     /* Remove invisible dummy row */
     last = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), parent);
@@ -209,23 +230,30 @@ midori_bookmarks_read_from_db_to_model (MidoriBookmarks* bookmarks,
         g_object_unref (item);
 }
 
-void
-midori_bookmarks_insert_item_db (sqlite3*     db,
-                                 KatzeItem*   item,
-                                const gchar* folder)
+gint64
+midori_bookmarks_insert_item_db (sqlite3*   db,
+                                 KatzeItem* item,
+                                 gint64     parentid)
 {
     gchar* sqlcmd;
     char* errmsg = NULL;
     KatzeItem* old_parent;
-    const gchar* parent;
+    gchar* new_parentid;
+    gchar* id = NULL;
     const gchar* uri = NULL;
     const gchar* desc = NULL;
+    gint64 seq = 0;
 
     /* Bookmarks must have a name, import may produce invalid items */
-    g_return_if_fail (katze_item_get_name (item));
+    g_return_val_if_fail (katze_item_get_name (item), seq);
 
     if (!db)
-        return;
+        return seq;
+
+    if (katze_item_get_meta_integer (item, "id") > 0)
+        id = g_strdup_printf ("%" G_GINT64_FORMAT, katze_item_get_meta_integer(item, "id"));
+    else
+        id = g_strdup_printf ("NULL");
 
     if (KATZE_ITEM_IS_BOOKMARK (item))
         uri = katze_item_get_uri (item);
@@ -235,30 +263,57 @@ midori_bookmarks_insert_item_db (sqlite3*     db,
 
     /* Use folder, otherwise fallback to parent folder */
     old_parent = katze_item_get_parent (item);
-    if (folder && *folder)
-        parent = folder;
-    else if (old_parent && katze_item_get_name (old_parent))
-        parent = katze_item_get_name (old_parent);
+    if (parentid > 0)
+        new_parentid = g_strdup_printf ("%" G_GINT64_FORMAT, parentid);
+    else if (old_parent && katze_item_get_meta_integer (old_parent, "id") > 0)
+        new_parentid = g_strdup_printf ("%" G_GINT64_FORMAT, katze_item_get_meta_integer (old_parent, "id"));
     else
-        parent = "";
+        new_parentid = g_strdup_printf ("NULL");
 
     sqlcmd = sqlite3_mprintf (
-            "INSERT into bookmarks (uri, title, desc, folder, toolbar, app) values"
-            " ('%q', '%q', '%q', '%q', %d, %d)",
-            uri ? uri : "",
+            "INSERT INTO bookmarks (id, parentid, title, uri, desc, toolbar, app) "
+            "VALUES (%q, %q, '%q', '%q', '%q', %d, %d)",
+            id,
+            new_parentid,
             katze_item_get_name (item),
-            desc ? desc : "",
-            parent,
+            katze_str_non_null (uri),
+            katze_str_non_null (desc),
             katze_item_get_meta_boolean (item, "toolbar"),
             katze_item_get_meta_boolean (item, "app"));
 
-    if (sqlite3_exec (db, sqlcmd, NULL, NULL, &errmsg) != SQLITE_OK)
+    if (sqlite3_exec (db, sqlcmd, NULL, NULL, &errmsg) == SQLITE_OK)
+    {
+        /* Get insert id */
+        if (g_str_equal (id, "NULL"))
+        {
+            KatzeArray* seq_array;
+
+            sqlite3_free (sqlcmd);
+            sqlcmd = sqlite3_mprintf (
+                    "SELECT seq FROM sqlite_sequence WHERE name = 'bookmarks'");
+
+            seq_array = katze_array_from_sqlite (db, sqlcmd);
+            if (katze_array_get_nth_item (seq_array, 0))
+            {
+                KatzeItem* seq_item = katze_array_get_nth_item (seq_array, 0);
+
+                seq = katze_item_get_meta_integer (seq_item, "seq");
+                katze_item_set_meta_integer (item, "id", seq);
+            }
+            g_object_unref (seq_array);
+        }
+    }
+    else
     {
         g_printerr (_("Failed to add bookmark item: %s\n"), errmsg);
         sqlite3_free (errmsg);
     }
 
     sqlite3_free (sqlcmd);
+    g_free (new_parentid);
+    g_free (id);
+
+    return seq;
 }
 
 static void
@@ -268,15 +323,9 @@ midori_bookmarks_add_item_cb (KatzeArray*      array,
 {
     GtkTreeModel* model;
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (bookmarks->treeview));
-    if (!g_strcmp0 (katze_item_get_meta_string (item, "folder"), ""))
-        gtk_tree_store_insert_with_values (GTK_TREE_STORE (model),
-                                           NULL, NULL, G_MAXINT, 0, item, -1);
-    else
-    {
-        gtk_tree_store_clear (GTK_TREE_STORE (model));
-        midori_bookmarks_read_from_db_to_model (bookmarks,
-            GTK_TREE_STORE (model), NULL, NULL, bookmarks->filter);
-    }
+    gtk_tree_store_clear (GTK_TREE_STORE (model));
+    midori_bookmarks_read_from_db_to_model (bookmarks,
+        GTK_TREE_STORE (model), NULL, 0, bookmarks->filter);
 }
 
 static void
@@ -287,7 +336,7 @@ midori_bookmarks_remove_item_cb (KatzeArray*      array,
     GtkTreeModel* model = gtk_tree_view_get_model (GTK_TREE_VIEW (bookmarks->treeview));
     gtk_tree_store_clear (GTK_TREE_STORE (model));
     midori_bookmarks_read_from_db_to_model (bookmarks,
-        GTK_TREE_STORE (model), NULL, NULL, bookmarks->filter);
+        GTK_TREE_STORE (model), NULL, 0, bookmarks->filter);
 }
 
 static void
@@ -297,7 +346,7 @@ midori_bookmarks_update_cb (KatzeArray*      array,
     GtkTreeModel* model = gtk_tree_view_get_model (GTK_TREE_VIEW (bookmarks->treeview));
     gtk_tree_store_clear (GTK_TREE_STORE (model));
     midori_bookmarks_read_from_db_to_model (bookmarks,
-        GTK_TREE_STORE (model), NULL, NULL, bookmarks->filter);
+        GTK_TREE_STORE (model), NULL, 0, bookmarks->filter);
 }
 
 
@@ -310,7 +359,7 @@ midori_bookmarks_row_changed_cb (GtkTreeModel*    model,
     KatzeItem* item;
     GtkTreeIter parent;
     KatzeItem* new_parent = NULL;
-    const gchar* parent_name;
+    gint64 parentid;
 
     gtk_tree_model_get (model, iter, 0, &item, -1);
 
@@ -320,15 +369,15 @@ midori_bookmarks_row_changed_cb (GtkTreeModel*    model,
 
         /* Bookmarks must not be moved into non-folder items */
         if (!KATZE_ITEM_IS_FOLDER (new_parent))
-            parent_name = "";
+            parentid = 0;
         else
-            parent_name = katze_item_get_name (new_parent);
+            parentid = katze_item_get_meta_integer (new_parent, "id");
     }
     else
-        parent_name = "";
+        parentid = 0;
 
     katze_array_remove_item (bookmarks->array, item);
-    katze_item_set_meta_string (item, "folder", parent_name);
+    katze_item_set_meta_integer (item, "parentid", parentid);
     katze_array_add_item (bookmarks->array, item);
 
     g_object_unref (item);
@@ -359,14 +408,24 @@ midori_bookmarks_edit_clicked_cb (GtkWidget*       toolitem,
     {
         KatzeItem* item;
         MidoriBrowser* browser;
+        gint64 parentid;
 
         gtk_tree_model_get (model, &iter, 0, &item, -1);
 
         g_assert (!KATZE_ITEM_IS_SEPARATOR (item));
 
         browser = midori_browser_get_for_widget (bookmarks->treeview);
+        parentid = katze_item_get_meta_integer (item, "parentid");
         midori_browser_edit_bookmark_dialog_new (
             browser, item, FALSE, KATZE_ITEM_IS_FOLDER (item), NULL);
+
+        if (katze_item_get_meta_integer (item, "parentid") != parentid)
+        {
+            gtk_tree_store_clear (GTK_TREE_STORE (model));
+            midori_bookmarks_read_from_db_to_model (bookmarks, GTK_TREE_STORE (model),
+                    NULL, 0, NULL);
+        }
+
         g_object_unref (item);
     }
 }
@@ -382,6 +441,47 @@ midori_bookmarks_toolbar_update (MidoriBookmarks *bookmarks)
     gtk_widget_set_sensitive (GTK_WIDGET (bookmarks->edit), selected);
 }
 
+gboolean
+midori_bookmarks_update_item_db (sqlite3*   db,
+                                 KatzeItem* item)
+{
+    gchar* sqlcmd;
+    char* errmsg = NULL;
+    gchar* parentid;
+    gboolean updated;
+
+    if (katze_item_get_meta_integer (item, "parentid") > 0)
+        parentid = g_strdup_printf ("%" G_GINT64_FORMAT,
+                                    katze_item_get_meta_integer (item, "parentid"));
+    else
+        parentid = g_strdup_printf ("NULL");
+
+    sqlcmd = sqlite3_mprintf (
+            "UPDATE bookmarks SET "
+            "parentid=%q, title='%q', uri='%q', desc='%q', toolbar=%d, app=%d "
+            "WHERE id = %" G_GINT64_FORMAT ";",
+            parentid,
+            katze_item_get_name (item),
+            katze_str_non_null (katze_item_get_uri (item)),
+            katze_str_non_null (katze_item_get_meta_string (item, "desc")),
+            katze_item_get_meta_boolean (item, "toolbar"),
+            katze_item_get_meta_boolean (item, "app"),
+            katze_item_get_meta_integer (item, "id"));
+
+    updated = TRUE;
+    if (sqlite3_exec (db, sqlcmd, NULL, NULL, &errmsg) != SQLITE_OK)
+    {
+        updated = FALSE;
+        g_printerr (_("Failed to update bookmark : %s\n"), errmsg);
+        sqlite3_free (errmsg);
+    }
+
+    sqlite3_free (sqlcmd);
+    g_free (parentid);
+
+    return updated;
+}
+
 static void
 midori_bookmarks_delete_clicked_cb (GtkWidget*       toolitem,
                                     MidoriBookmarks* bookmarks)
@@ -421,7 +521,7 @@ midori_bookmarks_get_toolbar (MidoriViewable* viewable)
         gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_BUTTON);
         gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
         bookmarks->toolbar = toolbar;
-        toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_ADD);
+        toolitem = gtk_tool_button_new_from_stock (STOCK_BOOKMARK_ADD);
         gtk_widget_set_name (GTK_WIDGET (toolitem), "BookmarkAdd");
         gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem),
                                      _("Add a new bookmark"));
@@ -452,7 +552,7 @@ midori_bookmarks_get_toolbar (MidoriViewable* viewable)
         gtk_tool_item_set_expand (toolitem, TRUE);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
         gtk_widget_show (GTK_WIDGET (toolitem));
-        toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_DIRECTORY);
+        toolitem = gtk_tool_button_new_from_stock (STOCK_FOLDER_NEW);
         gtk_widget_set_name (GTK_WIDGET (toolitem), "BookmarkFolderAdd");
         gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem),
                                      _("Add a new folder"));
@@ -496,8 +596,8 @@ midori_bookmarks_set_app (MidoriBookmarks* bookmarks,
 
     g_object_ref (app);
     bookmarks->array = katze_object_get_object (app, "bookmarks");
-    midori_bookmarks_read_from_db_to_model (bookmarks, GTK_TREE_STORE (model), NULL, "", NULL);
-    g_signal_connect (bookmarks->array, "add-item",
+    midori_bookmarks_read_from_db_to_model (bookmarks, GTK_TREE_STORE (model), NULL, 0, NULL);
+    g_signal_connect_after (bookmarks->array, "add-item",
                       G_CALLBACK (midori_bookmarks_add_item_cb), bookmarks);
     g_signal_connect (bookmarks->array, "remove-item",
                       G_CALLBACK (midori_bookmarks_remove_item_cb), bookmarks);
@@ -588,8 +688,11 @@ midori_bookmarks_treeview_render_text_cb (GtkTreeViewColumn* column,
     gtk_tree_model_get (model, iter, 0, &item, -1);
 
     if (item && katze_item_get_name (item))
+    {
         g_object_set (renderer, "markup", NULL,
+                      "ellipsize", PANGO_ELLIPSIZE_END,
                       "text", katze_item_get_name (item), NULL);
+    }
     else
         g_object_set (renderer, "markup", _("<i>Separator</i>"), NULL);
 
@@ -687,7 +790,9 @@ midori_bookmarks_open_in_tab_activate_cb (GtkWidget*       menuitem,
         KatzeItem* child;
         KatzeArray* array;
 
-        array = midori_bookmarks_read_from_db (bookmarks, katze_item_get_name (item), NULL);
+        array = midori_bookmarks_read_from_db (bookmarks,
+                katze_item_get_meta_integer (item, "parentid"), NULL);
+
         g_return_if_fail (KATZE_IS_ARRAY (array));
         KATZE_ARRAY_FOREACH_ITEM (child, array)
         {
@@ -843,7 +948,7 @@ midori_bookmarks_row_expanded_cb (GtkTreeView*     treeview,
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
     gtk_tree_model_get (model, iter, 0, &item, -1);
     midori_bookmarks_read_from_db_to_model (bookmarks, GTK_TREE_STORE (model),
-                                            iter, katze_item_get_name (item), NULL);
+                                            iter, katze_item_get_meta_integer (item, "id"), NULL);
     g_object_unref (item);
 }
 
@@ -885,7 +990,7 @@ midori_bookmarks_filter_timeout_cb (gpointer data)
 
     gtk_tree_store_clear (treestore);
     midori_bookmarks_read_from_db_to_model (bookmarks,
-        treestore, NULL, NULL, bookmarks->filter);
+        treestore, NULL, 0, bookmarks->filter);
 
     return FALSE;
 }
@@ -896,21 +1001,16 @@ midori_bookmarks_filter_entry_changed_cb (GtkEntry*        entry,
 {
     if (bookmarks->filter_timeout)
         g_source_remove (bookmarks->filter_timeout);
-    katze_assign (bookmarks->filter, g_strdup (gtk_entry_get_text (entry)));
+
+    if (!g_object_get_data (G_OBJECT (entry), "sokoke_has_default"))
+        katze_assign (bookmarks->filter, g_strdup (gtk_entry_get_text (entry)));
+    else
+        katze_assign (bookmarks->filter, NULL);
+
     bookmarks->filter_timeout = g_timeout_add (COMPLETION_DELAY,
         midori_bookmarks_filter_timeout_cb, bookmarks);
 }
 
-static void
-midori_bookmarks_filter_entry_clear_cb (GtkEntry*        entry,
-                                        gint             icon_pos,
-                                        gint             button,
-                                        MidoriBookmarks* bookmarks)
-{
-    if (icon_pos == GTK_ICON_ENTRY_SECONDARY)
-        gtk_entry_set_text (entry, "");
-}
-
 static void
 midori_bookmarks_init (MidoriBookmarks* bookmarks)
 {
@@ -924,14 +1024,8 @@ midori_bookmarks_init (MidoriBookmarks* bookmarks)
     GtkTreeSelection* selection;
 
     /* Create the filter entry */
-    entry = gtk_icon_entry_new ();
-    gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
-                                        GTK_ICON_ENTRY_PRIMARY,
-                                        GTK_STOCK_FIND);
-    sokoke_entry_set_clear_button_visible (GTK_ENTRY (entry), TRUE);
-    g_signal_connect (entry, "icon-release",
-        G_CALLBACK (midori_bookmarks_filter_entry_clear_cb), bookmarks);
-    g_signal_connect (entry, "changed",
+    entry = sokoke_search_entry_new (_("Search Bookmarks"));
+    g_signal_connect_after (entry, "changed",
         G_CALLBACK (midori_bookmarks_filter_entry_changed_cb), bookmarks);
     box = gtk_hbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 3);
@@ -944,14 +1038,14 @@ midori_bookmarks_init (MidoriBookmarks* bookmarks)
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
     gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (treeview), 1);
     column = gtk_tree_view_column_new ();
-    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+    gtk_tree_view_column_set_expand (column, TRUE);
     renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, renderer_pixbuf, FALSE);
     gtk_tree_view_column_set_cell_data_func (column, renderer_pixbuf,
         (GtkTreeCellDataFunc)midori_bookmarks_treeview_render_icon_cb,
         treeview, NULL);
     renderer_text = gtk_cell_renderer_text_new ();
-    gtk_tree_view_column_pack_start (column, renderer_text, FALSE);
+    gtk_tree_view_column_pack_start (column, renderer_text, TRUE);
     gtk_tree_view_column_set_cell_data_func (column, renderer_text,
         (GtkTreeCellDataFunc)midori_bookmarks_treeview_render_text_cb,
         treeview, NULL);
