@@ -245,9 +245,9 @@ int
 main (int    argc,
       char** argv)
 {
-    midori_app_setup (argv);
     g_test_init (&argc, &argv, NULL);
-    gtk_init_check (&argc, &argv);
+    midori_app_setup (&argc, &argv, NULL, NULL);
+    midori_paths_init (MIDORI_RUNTIME_MODE_PRIVATE, NULL);
     soup_session_add_feature_by_type (webkit_get_default_session (),
         SOUP_TYPE_COOKIE_JAR);
 
